@@ -27,6 +27,7 @@ import os
 from distutils import sysconfig
 from pybuilder.core import init, after, use_plugin
 from pybuilder.utils import discover_modules, render_report, fork_process, is_windows
+from pybuilder.coverage_utils import patch_multiprocessing, reverse_patch_multiprocessing
 from pybuilder.errors import BuildFailedException
 
 use_plugin("python.core")
@@ -35,12 +36,13 @@ use_plugin("analysis")
 
 @init
 def init_coverage_properties(project):
-    project.plugin_depends_on("coverage")
+    project.plugin_depends_on("coverage", "~=4.2")
 
     project.set_property_if_unset("coverage_threshold_warn", 70)
     project.set_property_if_unset("coverage_branch_threshold_warn", 0)
     project.set_property_if_unset("coverage_branch_partial_threshold_warn", 0)
     project.set_property_if_unset("coverage_break_build", True)
+    project.set_property_if_unset("coverage_allow_non_imported_modules", True)
     project.set_property_if_unset("coverage_reload_modules", None)  # deprecated, unused
     project.set_property_if_unset("coverage_reset_modules", False)
     project.set_property_if_unset("coverage_exceptions", [])
@@ -73,9 +75,9 @@ def run_coverage(project, logger, reactor, execution_prefix, execution_name, tar
     logger.debug("Forking process to do %s analysis", execution_name)
     exit_code, _ = fork_process(logger,
                                 target=do_coverage,
-                                args=(
-                                    project, logger, reactor, execution_prefix, execution_name,
-                                    target_task, shortest_plan))
+                                args=(project, logger, reactor, execution_prefix, execution_name, target_task,
+                                      shortest_plan))
+
     if exit_code and project.get_property("%s_break_build" % execution_prefix):
         raise BuildFailedException(
             "Forked %s process indicated failure with error code %d" % (execution_name, exit_code))
@@ -89,6 +91,7 @@ def do_coverage(project, logger, reactor, execution_prefix, execution_name, targ
     """
     source_tree_path = project.get_property("dir_source_main_python")
     reset_modules = project.get_property("%s_reset_modules" % execution_prefix)
+    allow_non_imported_modules = project.get_property("%s_allow_non_imported_modules" % execution_prefix)
     module_names = _discover_modules_to_cover(project)
 
     for module_name in module_names:
@@ -103,20 +106,29 @@ def do_coverage(project, logger, reactor, execution_prefix, execution_name, targ
 
     coverage = coverage_factory(cover_pylib=False, branch=True, source=[source_tree_path])
 
+    patch_multiprocessing(coverage.config)
     try:
-        _start_coverage(project, coverage)
+        try:
+            _start_coverage(project, coverage)
 
-        if shortest_plan:
-            reactor.execute_task_shortest_plan(target_task)
-        else:
-            reactor.execute_task(target_task)
+            if shortest_plan:
+                reactor.execute_task_shortest_plan(target_task)
+            else:
+                reactor.execute_task(target_task)
+        finally:
+            _stop_coverage(project, coverage)
     finally:
-        _stop_coverage(project, coverage)
+        reverse_patch_multiprocessing()
 
     module_exceptions = project.get_property("%s_exceptions" % execution_prefix)
-    modules = _list_all_covered_modules(logger, module_names, module_exceptions)
+    modules, non_imported_modules = _list_all_covered_modules(logger, module_names, module_exceptions,
+                                                              allow_non_imported_modules)
 
     failure = _build_coverage_report(project, logger, execution_name, execution_prefix, coverage, modules)
+
+    if non_imported_modules and not allow_non_imported_modules:
+        raise BuildFailedException("Some modules have not been imported and have no coverage")
+
     if failure:
         raise failure
 
@@ -131,10 +143,12 @@ def _start_coverage(project, coverage):
 def _stop_coverage(project, coverage):
     coverage.stop()
     project.set_property('__running_coverage', False)
+    coverage.combine()
 
 
-def _list_all_covered_modules(logger, module_names, modules_exceptions):
+def _list_all_covered_modules(logger, module_names, modules_exceptions, allow_non_imported_modules):
     modules = []
+    non_imported_modules = []
     for module_name in module_names:
         if module_name in modules_exceptions:
             logger.debug("Module '%s' was excluded", module_name)
@@ -142,17 +156,25 @@ def _list_all_covered_modules(logger, module_names, modules_exceptions):
         try:
             module = sys.modules[module_name]
         except KeyError:
-            logger.warn("Module '%s' was not imported by the covered tests", module_name)
+            non_imported_modules.append(module_name)
+            if allow_non_imported_modules:
+                logger_func = logger.warn
+            else:
+                logger_func = logger.error
+
+            logger_func("Module '%s' was not imported by the covered tests", module_name)
             try:
                 module = __import__(module_name)
             except SyntaxError as e:
-                logger.warn("Coverage for module '%s' cannot be established - syntax error: %s",
-                            module_name, e)
+                logger.warn("Coverage for module '%s' cannot be established - syntax error: %s", module_name, e)
+                continue
+            except Exception as e:
+                logger.warn("Coverage for module '%s' cannot be established - module failed: %s", module_name, e)
                 continue
 
         if module not in modules and hasattr(module, "__file__"):
             modules.append(module)
-    return modules
+    return modules, non_imported_modules
 
 
 def _build_module_report(coverage, module):
@@ -313,6 +335,10 @@ def _is_module_essential(module_name, sys_packages, sys_modules):
     if module_name in sys_modules:
         return True
 
+    # Issue 523: Coverage 4.4.2 breaks without this
+    if module_name == "__main__":
+        return True
+
     # Essential since we're in a fork for communicating exceptions back
     sys_packages.append("tblib")
     sys_packages.append("pybuilder.errors")
