@@ -111,7 +111,7 @@ def as_str(value):
 
 @init
 def initialize_distutils_plugin(project):
-    project.plugin_depends_on("pypandoc", "~=1.3.0")
+    project.plugin_depends_on("pypandoc")
 
     project.set_property_if_unset("distutils_commands", ["sdist", "bdist_wheel"])
     project.set_property_if_unset("distutils_command_options", None)
@@ -143,8 +143,8 @@ def set_description(project, logger):
         try:
             assert_can_execute(["pandoc", "--version"], "pandoc", "distutils")
             doc_convert(project, logger)
-        except MissingPrerequisiteException:
-            logger.warn("Was unable to find pandoc and did not convert the documentation")
+        except (MissingPrerequisiteException, ImportError):
+            logger.warn("Was unable to find pandoc or pypandoc and did not convert the documentation")
 
 
 @after("package")
@@ -409,7 +409,7 @@ def build_scripts_string(project):
 
     scripts_dir = project.get_property("dir_dist_scripts")
     if scripts_dir:
-        scripts = list(map(lambda s: os.path.join(scripts_dir, s), scripts))
+        scripts = list(map(lambda s: '%s/%s' % (scripts_dir, s), scripts))
 
     return build_string_from_array(scripts)
 
